@@ -52,11 +52,13 @@ class BaseAgentConnection(ABC):
             timeout=timeout,
             loop=loop
         )
+        self.__loop = loop or asyncio.get_event_loop()
         self._p2p = p2p
         self._timeout = timeout
 
     def __del__(self):
-        asyncio.ensure_future(self.close())
+        if self.__loop and self.__loop.is_running():
+            asyncio.ensure_future(self.close())
 
     @property
     def timeout(self):
@@ -196,7 +198,7 @@ class AgentRPC(BaseAgentConnection):
             self, message: Message,
             their_vk: Union[List[str], str], endpoint: str,
             my_vk: Optional[str], routing_keys: Optional[List[str]],
-            coprotocol: bool = False
+            coprotocol: bool = False, ignore_errors: bool = False
     ) -> Optional[Message]:
         """Send Message to other Indy compatible agent
         
@@ -209,6 +211,7 @@ class AgentRPC(BaseAgentConnection):
             See:
              - https://github.com/hyperledger/aries-rfcs/tree/master/concepts/0003-protocols
              - https://github.com/hyperledger/aries-rfcs/tree/master/concepts/0008-message-id-and-threading
+        :param ignore_errors: bool hide any raised exception
         :return: Response message if coprotocol is True
         """
         if not self._connector.is_open:
@@ -243,7 +246,8 @@ class AgentRPC(BaseAgentConnection):
                 ok, body = await http_send(wired, endpoint, timeout=self.timeout, connector=self.__tcp_connector)
             body = body.decode()
         if not ok:
-            raise SiriusRPCError(body)
+            if not ignore_errors:
+                raise SiriusRPCError(body)
         else:
             if coprotocol:
                 response = await self.read_protocol_message()
